@@ -36,9 +36,6 @@ as that of the covered work.  */
 #include <unistd.h>
 #include <errno.h>
 #include <assert.h>
-#ifdef ENABLE_THREADS
-#include <pthread.h>
-#endif
 #include "convert.h"
 #include "url.h"
 #include "recur.h"
@@ -58,49 +55,6 @@ struct hash_table *dl_url_file_map;
 struct hash_table *downloaded_html_set;
 struct hash_table *downloaded_css_set;
 
-#ifdef ENABLE_THREADS
-static pthread_mutex_t convert_mutex = PTHREAD_MUTEX_INITIALIZER;
-
-static void
-lock_convert_mutex()
-{
-  static int convert_mutex_init_p = 0;
-  if (! convert_mutex_init_p)
-    {
-      pthread_mutexattr_t mta;
-      pthread_mutexattr_init (&mta);
-      pthread_mutexattr_settype (&mta, PTHREAD_MUTEX_RECURSIVE);
-
-      pthread_mutex_init (&convert_mutex, &mta);
-      convert_mutex_init_p = 1;
-    }
-  pthread_mutex_lock (&convert_mutex);
-}
-
-
-#define THREAD_SAFE(ret, fn, args, argv)          \
-  ret fn args                                     \
-  {                                               \
-    ret r;                                        \
-    lock_convert_mutex();                         \
-    r = fn##_1 argv;                              \
-    pthread_mutex_unlock (&convert_mutex);        \
-    return r;                                     \
-  }
-
-#define THREAD_SAFE_VOID(fn, args, argv)            \
-  void fn args                                      \
-  {                                                 \
-    lock_convert_mutex();                           \
-    fn##_1 argv;                                    \
-    pthread_mutex_unlock (&convert_mutex);          \
-  }
-
-#define FNNAME_WTHREADS(fn) fn##_1
-#else
-#define FNNAME_WTHREADS(fn) fn
-#endif
-
 static void convert_links (const char *, struct urlpos *);
 
 
@@ -228,7 +182,7 @@ convert_links_in_hashtable (struct hash_table *downloaded_set,
    extracted from these two lists.  */
 
 void
-FNNAME_WTHREADS(convert_all_links) (void)
+convert_all_links (void)
 {
   double secs;
   int file_count = 0;
@@ -806,7 +760,7 @@ dissociate_urls_from_file (const char *file)
    URL has already been downloaded.  */
 
 void
-FNNAME_WTHREADS(register_download) (const char *url, const char *file)
+register_download (const char *url, const char *file)
 {
   char *old_file, *old_url;
 
@@ -886,7 +840,7 @@ FNNAME_WTHREADS(register_download) (const char *url, const char *file)
    register_download() above.  */
 
 void
-FNNAME_WTHREADS(register_redirection) (const char *from, const char *to)
+register_redirection (const char *from, const char *to)
 {
   char *file;
 
@@ -901,7 +855,7 @@ FNNAME_WTHREADS(register_redirection) (const char *from, const char *to)
 /* Register that the file has been deleted. */
 
 void
-FNNAME_WTHREADS(register_delete_file) (const char *file)
+register_delete_file (const char *file)
 {
   char *old_url, *old_file;
 
@@ -919,7 +873,7 @@ FNNAME_WTHREADS(register_delete_file) (const char *file)
 /* Register that FILE is an HTML file that has been downloaded. */
 
 void
-FNNAME_WTHREADS(register_html) (const char *url, const char *file)
+register_html (const char *file)
 {
   if (!downloaded_html_set)
     downloaded_html_set = make_string_hash_table (0);
@@ -929,7 +883,7 @@ FNNAME_WTHREADS(register_html) (const char *url, const char *file)
 /* Register that FILE is a CSS file that has been downloaded. */
 
 void
-FNNAME_WTHREADS(register_css) (const char *url, const char *file)
+register_css (const char *file)
 {
   if (!downloaded_css_set)
     downloaded_css_set = make_string_hash_table (0);
@@ -940,8 +894,8 @@ static void downloaded_files_free (void);
 
 /* Cleanup the data structures associated with this file.  */
 
-static void
-FNNAME_WTHREADS(convert_cleanup) (void)
+void
+convert_cleanup (void)
 {
   if (dl_file_url_map)
     {
@@ -1016,7 +970,7 @@ downloaded_mode_to_ptr (downloaded_file_t mode)
    URLs.  */
 
 downloaded_file_t
-FNNAME_WTHREADS(downloaded_file) (downloaded_file_t mode, const char *file)
+downloaded_file (downloaded_file_t mode, const char *file)
 {
   downloaded_file_t *ptr;
 
@@ -1130,17 +1084,6 @@ html_quote_string (const char *s)
   return res;
 }
 
-#ifdef ENABLE_THREADS
-THREAD_SAFE (downloaded_file_t, downloaded_file, (downloaded_file_t a, const char *b), (a, b));
-THREAD_SAFE_VOID (register_download, (const char *a, const char *b), (a, b));
-THREAD_SAFE_VOID (register_redirection, (const char *a, const char *b), (a, b));
-THREAD_SAFE_VOID (register_html, (const char *a, const char *b), (a, b));
-THREAD_SAFE_VOID (register_css, (const char *a, const char *b), (a, b));
-THREAD_SAFE_VOID (register_delete_file, (const char *a), (a));
-THREAD_SAFE_VOID (convert_cleanup, (void), ());
-THREAD_SAFE_VOID (convert_all_links, (void), ());
-#endif
-
 /*
  * vim: et ts=2 sw=2
  */
