@@ -32,8 +32,6 @@ as that of the covered work.  */
 #ifndef FTP_H
 #define FTP_H
 
-#include "wget.h"
-
 #include "host.h"
 
 /* System types. */
@@ -47,6 +45,14 @@ enum stype
   ST_OTHER
 };
 
+/* Extensions of the ST_UNIX */
+enum ustype
+{
+  UST_TYPE_L8,
+  UST_MULTINET,
+  UST_OTHER
+};
+
 extern char ftp_last_respline[];
 
 uerr_t ftp_response (int, char **);
@@ -63,8 +69,8 @@ uerr_t ftp_type (int, int);
 uerr_t ftp_cwd (int, const char *);
 uerr_t ftp_retr (int, const char *);
 uerr_t ftp_rest (int, wgint);
-uerr_t ftp_list (int, const char *, enum stype);
-uerr_t ftp_syst (int, enum stype *);
+uerr_t ftp_list (int, const char *, bool, bool, bool *);
+uerr_t ftp_syst (int, enum stype *, enum ustype *);
 uerr_t ftp_pwd (int, char **);
 uerr_t ftp_size (int, const char *, wgint *);
 
@@ -126,12 +132,27 @@ enum wget_ftp_fstatus
   NOTHING       = 0x0000,	/* Nothing done yet.  */
   ON_YOUR_OWN   = 0x0001,	/* The ftp_loop_internal sets the
 				   defaults.  */
-  DONE_CWD      = 0x0002	/* The current working directory is
+  DONE_CWD      = 0x0002,	/* The current working directory is
 				   correct.  */
+
+  /* 2013-10-17 Andrea Urbani (matfanjol)
+     For more information about the following entries, please,
+     look at ftp.c, function getftp, text "__LIST_A_EXPLANATION__". */
+  AVOID_LIST_A  = 0x0004,	/* It tells us if during this
+				 session we have to avoid the use
+				 of "LIST -a".*/
+  AVOID_LIST    = 0x0008,	/* It tells us if during this
+				 session we have to avoid to use
+				 "LIST". */
+  LIST_AFTER_LIST_A_CHECK_DONE  = 0x0010
+				/* It tells us if we have already
+				 checked "LIST" after the first
+				 "LIST -a" to handle the case of
+				 file/folders named "-a". */
 };
 
 struct fileinfo *ftp_parse_ls (const char *, const enum stype);
-uerr_t ftp_loop (struct url *, char **, int *, struct url *, bool, bool, struct range *);
+uerr_t ftp_loop (struct url *, char **, int *, struct url *, bool, bool);
 
 uerr_t ftp_index (const char *, struct url *, struct fileinfo *);
 
