@@ -35,11 +35,6 @@ struct options
                                    hence not boolean.) */
   bool quiet;			/* Are we quiet? */
   int ntry;			/* Number of tries per URL */
-#ifdef ENABLE_METALINK
-  int n_retries;		/* Number of tries per file */
-#endif
-  char *metalink_file;		/* Metalink filename */
-
   bool retry_connrefused;	/* Treat CONNREFUSED as non-fatal. */
   bool background;		/* Whether we should work in background. */
   bool ignore_length;		/* Do we heed content-length at all?  */
@@ -171,7 +166,7 @@ struct options
   bool timestamping;		/* Whether to use time-stamping. */
 
   bool backup_converted;	/* Do we save pre-converted files as *.orig? */
-  bool backups;			/* Are numeric backups made? */
+  int backups;			/* Are numeric backups made? */
 
   char *useragent;		/* User-Agent string, which can be set
 				   to something other than Wget. */
@@ -205,7 +200,8 @@ struct options
     secure_protocol_auto,
     secure_protocol_sslv2,
     secure_protocol_sslv3,
-    secure_protocol_tlsv1
+    secure_protocol_tlsv1,
+    secure_protocol_pfs
   } secure_protocol;		/* type of secure protocol to use. */
   bool check_cert;		/* whether to validate the server's cert */
   char *cert_file;		/* external client certificate to use. */
@@ -220,9 +216,9 @@ struct options
   char *ca_directory;		/* CA directory (hash files) */
   char *ca_cert;		/* CA certificate file to use */
 
-
   char *random_file;		/* file with random data to seed the PRNG */
   char *egd_file;		/* file name of the egd daemon socket */
+  bool https_only;		/* whether to follow HTTPS only */
 #endif /* HAVE_SSL */
 
   bool cookies;			/* whether cookies are used. */
@@ -287,12 +283,7 @@ struct options
 
   bool show_all_dns_entries; /* Show all the DNS entries when resolving a
                                 name. */
-
   bool report_bps;              /*Output bandwidth in bits format*/
-
-#ifdef ENABLE_THREADS
-  int jobs; /* How many threads use at the same time.  */
-#endif
 };
 
 extern struct options opt;
