@@ -37,19 +37,10 @@ as that of the covered work.  */
 #include <errno.h>
 #include <string.h>
 #include <assert.h>
-#ifdef ENABLE_THREADS
-#include <pthread.h>
-#include <semaphore.h>
+#ifdef VMS
+# include <unixio.h>            /* For delete(). */
 #endif
-#ifdef ENABLE_METALINK
-#include <metalink/metalink_parser.h>
-#include <metalink/metalink_types.h>
 
-#include "metalink.h"
-#endif
-#ifdef ENABLE_THREADS
-#include "multi.h"
-#endif
 #include "exits.h"
 #include "utils.h"
 #include "retr.h"
@@ -66,17 +57,6 @@ as that of the covered work.  */
 #include "html-url.h"
 #include "iri.h"
 
-#ifdef ENABLE_METALINK
-static pthread_mutex_t pconn_mutex = PTHREAD_MUTEX_INITIALIZER;
-
-#define IS_IO_ERROR(status) status == FOPENERR || status == WRITEFAILED || \
-          status == UNLINKERR || status == FWRITEERR || status == FOPEN_EXCL_ERR
-
-#define PCONN_LOCK() pthread_mutex_lock (&pconn_mutex)
-
-#define PCONN_UNLOCK() pthread_mutex_unlock (&pconn_mutex)
-#endif
-
 /* Total size of downloaded files.  Used to enforce quota.  */
 SUM_SIZE_INT total_downloaded_bytes;
 
@@ -700,23 +680,23 @@ calc_rate (wgint bytes, double secs, int *units)
 }
 
 
-#define SUSPEND_POST_DATA do {                  \
-  post_data_suspended = true;                   \
-  saved_post_data = opt.body_data;              \
-  saved_post_file_name = opt.body_file;         \
+#define SUSPEND_METHOD do {                     \
+  method_suspended = true;                      \
+  saved_body_data = opt.body_data;              \
+  saved_body_file_name = opt.body_file;         \
   saved_method = opt.method;                    \
   opt.body_data = NULL;                         \
   opt.body_file = NULL;                         \
   opt.method = NULL;                            \
 } while (0)
 
-#define RESTORE_POST_DATA do {                          \
-  if (post_data_suspended)                              \
+#define RESTORE_METHOD do {                             \
+  if (method_suspended)                                 \
     {                                                   \
-      opt.body_data = saved_post_data;                  \
-      opt.body_file = saved_post_file_name;             \
+      opt.body_data = saved_body_data;                  \
+      opt.body_file = saved_body_file_name;             \
       opt.method = saved_method;                        \
-      post_data_suspended = false;                      \
+      method_suspended = false;                         \
     }                                                   \
 } while (0)
 
@@ -731,7 +711,7 @@ static char *getproxy (struct url *);
 uerr_t
 retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
               char **newloc, const char *refurl, int *dt, bool recursive,
-              struct iri *iri, bool register_status, struct range *segment_range)
+              struct iri *iri, bool register_status)
 {
   uerr_t result;
   char *url;
@@ -744,10 +724,10 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
   char *local_file;
   int redirection_count = 0;
 
-  bool post_data_suspended = false;
-  char *saved_post_data = NULL;
+  bool method_suspended = false;
+  char *saved_body_data = NULL;
   char *saved_method = NULL;
-  char *saved_post_file_name = NULL;
+  char *saved_body_file_name = NULL;
 
   /* If dt is NULL, use local storage.  */
   if (!dt)
@@ -758,13 +738,8 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
   url = xstrdup (origurl);
   if (newloc)
     *newloc = NULL;
-
-#ifndef ENABLE_METALINK
-  /* Note that, each and every call to retrieve_url(), except the ones made by
-     functions relevant to metalink support, the value of *file is NULL. */
   if (file)
     *file = NULL;
-#endif
 
   if (!refurl)
     refurl = opt.referer;
@@ -774,7 +749,7 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
 
   result = NOCONERROR;
   mynewloc = NULL;
-  local_file = *file;
+  local_file = NULL;
   proxy_url = NULL;
 
   proxy = getproxy (u);
@@ -793,7 +768,7 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
                      proxy, error);
           xfree (url);
           xfree (error);
-          RESTORE_POST_DATA;
+          RESTORE_METHOD;
           result = PROXERR;
           goto bail;
         }
@@ -802,7 +777,7 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
           logprintf (LOG_NOTQUIET, _("Error in proxy URL %s: Must be HTTP.\n"), proxy);
           url_free (proxy_url);
           xfree (url);
-          RESTORE_POST_DATA;
+          RESTORE_METHOD;
           result = PROXERR;
           goto bail;
         }
@@ -815,7 +790,7 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
       || (proxy_url && proxy_url->scheme == SCHEME_HTTP))
     {
       result = http_loop (u, orig_parsed, &mynewloc, &local_file, refurl, dt,
-                          proxy_url, iri, segment_range);
+                          proxy_url, iri);
     }
   else if (u->scheme == SCHEME_FTP)
     {
@@ -826,7 +801,7 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
       if (redirection_count)
         oldrec = glob = false;
 
-      result = ftp_loop (u, &local_file, dt, proxy_url, recursive, glob, segment_range);
+      result = ftp_loop (u, &local_file, dt, proxy_url, recursive, glob);
       recursive = oldrec;
 
       /* There is a possibility of having HTTP being redirected to
@@ -886,7 +861,7 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
           xfree (url);
           xfree (mynewloc);
           xfree (error);
-          RESTORE_POST_DATA;
+          RESTORE_METHOD;
           goto bail;
         }
 
@@ -908,7 +883,7 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
             }
           xfree (url);
           xfree (mynewloc);
-          RESTORE_POST_DATA;
+          RESTORE_METHOD;
           result = WRONGCODE;
           goto bail;
         }
@@ -931,8 +906,8 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
 	 RFC2616 HTTP/1.1 introduces code 307 Temporary Redirect
 	 specifically to preserve the method of the request.
 	 */
-      if (result != NEWLOCATION_KEEP_POST && !post_data_suspended)
-        SUSPEND_POST_DATA;
+      if (result != NEWLOCATION_KEEP_POST && !method_suspended)
+        SUSPEND_METHOD;
 
       goto redirected;
     }
@@ -965,10 +940,10 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
         register_redirection (origurl, u->url);
 
       if (*dt & TEXTHTML)
-        register_html ("",local_file);
+        register_html (local_file);
 
       if (*dt & TEXTCSS)
-        register_css ("",local_file);
+        register_css (local_file);
     }
 
   if (file)
@@ -995,7 +970,7 @@ retrieve_url (struct url * orig_parsed, const char *origurl, char **file,
       xfree (url);
     }
 
-  RESTORE_POST_DATA;
+  RESTORE_METHOD;
 
 bail:
   if (register_status)
@@ -1043,7 +1018,7 @@ retrieve_from_file (const char *file, bool html, int *count)
         opt.base_href = xstrdup (url);
 
       status = retrieve_url (url_parsed, url, &url_file, NULL, NULL, &dt,
-                             false, iri, true, NULL);
+                             false, iri, true);
       url_free (url_parsed);
 
       if (!url_file || (status != RETROK))
@@ -1067,293 +1042,69 @@ retrieve_from_file (const char *file, bool html, int *count)
   else
     input_file = (char *) file;
 
-#ifdef ENABLE_METALINK
-  mlink *mlink = parse_metalink(input_file);
+  url_list = (html ? get_urls_html (input_file, NULL, NULL, iri)
+              : get_urls_file (input_file));
 
-  if(opt.metalink_file && mlink)
+  xfree_null (url_file);
+
+  for (cur_url = url_list; cur_url; cur_url = cur_url->next, ++*count)
     {
-      int i, j, r, ranges_covered, chunk_size, url_err, retries, ret, dt=0;
-      pthread_t thread;
-      sem_t retr_sem;
-      uerr_t status;
-      mlink_file* file;
-      mlink_resource *resource;
-      struct s_thread_ctx *thread_ctx;
+      char *filename = NULL, *new_file = NULL;
+      int dt;
+      struct iri *tmpiri = iri_dup (iri);
+      struct url *parsed_url = NULL;
 
-      /* Wget supports HTTP&FTP, and Metalink supports MD5, SHA1 & SHA-256. */
-      elect_resources(mlink);
-      elect_checksums(mlink);
+      if (cur_url->ignore_when_downloading)
+        continue;
 
-      init_temp_files();
-      init_ranges ();
-      thread_ctx = malloc (opt.jobs * (sizeof *thread_ctx));
-      
-      retries = 0;
-      file = mlink->files;
-      while (file)
+      if (opt.quota && total_downloaded_bytes > opt.quota)
         {
-          memset(thread_ctx, '\0', opt.jobs * (sizeof *thread_ctx));
-
-          /* If chunk_size is too small, set it equal to MIN_CHUNK_SIZE. */
-          chunk_size = (file->size) / opt.jobs;
-          if(chunk_size < MIN_CHUNK_SIZE)
-            chunk_size = MIN_CHUNK_SIZE;
-          
-          j = fill_ranges_data(file->num_of_res, file->size, chunk_size);
-
-          /* If chunk_size was set to MIN_CHUNK_SIZE, opt.jobs should be corrected. */
-          if(j < opt.jobs)
-            opt.jobs = j;
-
-          name_temp_files();
-
-          sem_init (&retr_sem, 0, 0);
-          j = ranges_covered = 0;
-          resource = file->resources;
-          
-          /* Assign values to thread_ctx[] elements and spawn threads that will
-             conduct the download. */
-          for (r = 0; r < opt.jobs; ++r)
-            {
-              if (!resource)
-                {
-                  j = 0;
-                  resource = file->resources;
-                }
-
-              thread_ctx[r].referer = NULL;
-              thread_ctx[r].redirected = NULL;
-              thread_ctx[r].dt = dt;
-              thread_ctx[r].i = iri;
-              thread_ctx[r].url = resource->url;
-              thread_ctx[r].retr_sem = &retr_sem;
-
-              ret = spawn_thread (thread_ctx, r, j);
-              if (ret)
-                {
-                  /* If thread creation is unsuccessful */
-                  char *error = url_error (thread_ctx[r].url, thread_ctx[r].url_err);
-                  logprintf (LOG_NOTQUIET, "%s: %s.\n", thread_ctx[r].url, error);
-                  xfree (error);
-                  free(thread_ctx);
-                  clean_range_res_data();
-                  clean_ranges ();
-                  clean_temp_files ();
-                  return URLERROR;
-                }
-              ++j;
-              resource = resource->next;
-            }
-
-          /* Until all the ranges are covered, collect threads. */
-          while (ranges_covered < opt.jobs)
-            {
-              r = collect_thread (&retr_sem, thread_ctx);
-              ++ranges_covered;
-              
-              status = thread_ctx[r].status;
-
-              /* Check return status of thread for errors. */
-              if (IS_IO_ERROR(status))
-                {
-                  /* The error is of type WGET_EXIT_IO_FAIL given in exits.c.
-                     No fallbacking is needed for this type of error. */
-                  inform_exit_status(status);
-                  break;
-                }
-              else if(status != RETROK)
-                {
-                  int error_severity;
-                  PCONN_LOCK ();
-
-                  /* Pick the least severe error.*/
-                  error_severity = get_exit_status();
-                  inform_exit_status((thread_ctx[r].range)->status_least_severe);
-                  if(get_exit_status() != error_severity)
-                    (thread_ctx[r].range)->status_least_severe = status;
-                  PCONN_UNLOCK ();
-
-                  /* Look for resource from which downloading this range is not
-                     tried. */
-                  j = 0;
-                  resource = file->resources;
-                  while (j < file->num_of_res)
-                    {
-                      if (!((thread_ctx[r].range)->resources)[j])
-                        break;
-                      ++j;
-                      resource = resource -> next;
-                    }
-                  /* If there is such a resource, then update the range values
-                     to try that not-tried resource and spawn thread.
-                     If all the resources are exhausted, stop collecting the
-                     threads, as the download failed. */
-                  if (j < file->num_of_res)
-                    {
-                      if ((thread_ctx[r].range)->bytes_covered)
-                        {
-                          thread_ctx[r].url = resource->url;
-                          (thread_ctx[r].range)->first_byte =
-                                            (thread_ctx[r].range)->bytes_covered;
-                          (thread_ctx[r].range)->bytes_covered = 0;
-                        }
-                      --ranges_covered;
-                      ret = spawn_thread (thread_ctx, r, j);
-                      if (ret)
-                        {
-                          /* If thread creation is unsuccessful */
-                          char *error = url_error (thread_ctx[r].url, thread_ctx[r].url_err);
-                          logprintf (LOG_NOTQUIET, "%s: %s.\n", thread_ctx[r].url, error);
-                          xfree (error);
-                          free(thread_ctx);
-                          clean_range_res_data();
-                          clean_ranges ();
-                          clean_temp_files ();
-                          return URLERROR;
-                        }
-                    }
-                  else
-                    break;
-                }
-            }
-
-          sem_destroy(&retr_sem);
-
-          /* Check the download status. If conditions are suitable, retry. */
-          if (status != RETROK)
-            {
-              logprintf (LOG_VERBOSE, "Downloading %s failed. Chunk %d could not be downloaded from any of the URLs listed in metalink file.\n", file->name, r);
-
-              /* Failed downloads should only be retried if the error causing
-                 the failure is not an IO error. */
-              if (!(IS_IO_ERROR((thread_ctx[r].range)->status_least_severe)))
-                {
-                  if(retries  < opt.n_retries)
-                    {
-                      logprintf (LOG_VERBOSE, "Retrying to download(%s). (TRY #%d)\n",
-                                 file->name, ++retries + 1);
-                      continue;
-                    }
-                }
-            }
-          else
-            {
-              char *file_path;
-              int res;
-              /* Form the actual file to be downloaded and verify hash. */
-              file_path = malloc((opt.dir_prefix ? strlen(opt.dir_prefix) : 0)
-                               + strlen(file->name) + (sizeof "/"));
-              if(opt.dir_prefix)
-                sprintf(file_path, "%s/%s", opt.dir_prefix, file->name);
-              else
-                sprintf(file_path, "%s", file->name);
-              mkalldirs(file_path);
-              merge_temp_files(file_path);
-              res = verify_file_hash(file_path, file->checksums);
-              free(file_path);
-              if(!res)
-                {
-                  ++*count;
-                  logprintf (LOG_VERBOSE, "Verifying(%s) succeeded.\n",
-                               file->name);
-                }
-              else if(res < 0)
-                {
-                  logprintf (LOG_VERBOSE, "Verifying(%s) failed.\n", file->name);
-                  if(retries  < opt.n_retries)
-                    {
-                      logprintf (LOG_VERBOSE, "Retrying to download(%s). (TRY #%d)\n",
-                                 file->name, ++retries + 1);
-                      continue;
-                    }
-                }
-            }
-
-          delete_temp_files();
-
-          clean_range_res_data();
-          if (opt.quota && total_downloaded_bytes > opt.quota)
-            {
-              status = QUOTEXC;
-              break;
-            }
-          file = file->next;
+          status = QUOTEXC;
+          break;
         }
 
-      free(thread_ctx);
-      clean_ranges ();
-      clean_temp_files ();
-      delete_mlink(mlink);
-    }
-  else
-    {
-#endif
-      url_list = (html ? get_urls_html (input_file, NULL, NULL, iri)
-                  : get_urls_file (input_file));
+      parsed_url = url_parse (cur_url->url->url, NULL, tmpiri, true);
 
-      xfree_null (url_file);
-
-      for (cur_url = url_list; cur_url; cur_url = cur_url->next, ++*count)
+      if ((opt.recursive || opt.page_requisites)
+          && (cur_url->url->scheme != SCHEME_FTP || getproxy (cur_url->url)))
         {
-          char *filename = NULL, *new_file = NULL;
-          int dt;
-          struct iri *tmpiri = iri_dup (iri);
-          struct url *parsed_url = NULL;
+          int old_follow_ftp = opt.follow_ftp;
 
-          if (cur_url->ignore_when_downloading)
-            continue;
+          /* Turn opt.follow_ftp on in case of recursive FTP retrieval */
+          if (cur_url->url->scheme == SCHEME_FTP)
+            opt.follow_ftp = 1;
 
-          if (opt.quota && total_downloaded_bytes > opt.quota)
-            {
-              status = QUOTEXC;
-              break;
-            }
-
-          parsed_url = url_parse (cur_url->url->url, NULL, tmpiri, true);
-
-          if ((opt.recursive || opt.page_requisites)
-              && (cur_url->url->scheme != SCHEME_FTP || getproxy (cur_url->url)))
-            {
-              int old_follow_ftp = opt.follow_ftp;
-
-              /* Turn opt.follow_ftp on in case of recursive FTP retrieval */
-              if (cur_url->url->scheme == SCHEME_FTP)
-                opt.follow_ftp = 1;
-
-              status = retrieve_tree (parsed_url ? parsed_url : cur_url->url,
+          status = retrieve_tree (parsed_url ? parsed_url : cur_url->url,
                                   tmpiri);
 
-              opt.follow_ftp = old_follow_ftp;
-            }
-          else
-            status = retrieve_url (parsed_url ? parsed_url : cur_url->url,
-                                   cur_url->url->url, &filename,
-                                   &new_file, NULL, &dt, opt.recursive, tmpiri,
-                                   true, NULL);
+          opt.follow_ftp = old_follow_ftp;
+        }
+      else
+        status = retrieve_url (parsed_url ? parsed_url : cur_url->url,
+                               cur_url->url->url, &filename,
+                               &new_file, NULL, &dt, opt.recursive, tmpiri,
+                               true);
 
-          if (parsed_url)
-              url_free (parsed_url);
+      if (parsed_url)
+          url_free (parsed_url);
 
-          if (filename && opt.delete_after && file_exists_p (filename))
-            {
-              DEBUGP (("\
+      if (filename && opt.delete_after && file_exists_p (filename))
+        {
+          DEBUGP (("\
 Removing file due to --delete-after in retrieve_from_file():\n"));
-              logprintf (LOG_VERBOSE, _("Removing %s.\n"), filename);
-              if (unlink (filename))
-                logprintf (LOG_NOTQUIET, "unlink: %s\n", strerror (errno));
-              dt &= ~RETROKF;
-            }
-
-          xfree_null (new_file);
-          xfree_null (filename);
-          iri_free (tmpiri);
+          logprintf (LOG_VERBOSE, _("Removing %s.\n"), filename);
+          if (unlink (filename))
+            logprintf (LOG_NOTQUIET, "unlink: %s\n", strerror (errno));
+          dt &= ~RETROKF;
         }
 
-      /* Free the linked list of URL-s.  */
-      free_urlpos (url_list);
-#ifdef ENABLE_METALINK
+      xfree_null (new_file);
+      xfree_null (filename);
+      iri_free (tmpiri);
     }
-#endif
+
+  /* Free the linked list of URL-s.  */
+  free_urlpos (url_list);
 
   iri_free (iri);
 
@@ -1434,7 +1185,16 @@ free_urlpos (struct urlpos *l)
 void
 rotate_backups(const char *fname)
 {
-  int maxlen = strlen (fname) + 1 + numdigit (opt.backups) + 1;
+#ifdef __VMS
+# define SEP "_"
+# define AVS ";*"                       /* All-version suffix. */
+# define AVSL (sizeof (AVS) - 1)
+#else
+# define SEP "."
+# define AVSL 0
+#endif
+
+  int maxlen = strlen (fname) + sizeof (SEP) + numdigit (opt.backups) + AVSL;
   char *from = (char *)alloca (maxlen);
   char *to = (char *)alloca (maxlen);
   struct_stat sb;
@@ -1446,12 +1206,24 @@ rotate_backups(const char *fname)
 
   for (i = opt.backups; i > 1; i--)
     {
-      sprintf (from, "%s.%d", fname, i - 1);
-      sprintf (to, "%s.%d", fname, i);
+#ifdef VMS
+      /* Delete (all versions of) any existing max-suffix file, to avoid
+       * creating multiple versions of it.  (On VMS, rename() will
+       * create a new version of an existing destination file, not
+       * destroy/overwrite it.)
+       */
+      if (i == opt.backups)
+        {
+          sprintf (to, "%s%s%d%s", fname, SEP, i, AVS);
+          delete (to);
+        }
+#endif
+      sprintf (to, "%s%s%d", fname, SEP, i);
+      sprintf (from, "%s%s%d", fname, SEP, i - 1);
       rename (from, to);
     }
 
-  sprintf (to, "%s.%d", fname, 1);
+  sprintf (to, "%s%s%d", fname, SEP, 1);
   rename(fname, to);
 }
 
