@@ -357,20 +357,10 @@ typedef enum
   PROXERR,
   /* 50  */
   AUTHFAILED, QUOTEXC, WRITEFAILED, SSLINITFAILED, VERIFCERTERR,
-  UNLINKERR, NEWLOCATION_KEEP_POST, CLOSEFAILED, WARC_ERR,
-  WARC_TMP_FOPENERR,
-  /* 60  */
-  WARC_TMP_FWRITEERR, THREADS_ERR, SEM_ERR
-} uerr_t;
+  UNLINKERR, NEWLOCATION_KEEP_POST, CLOSEFAILED, ATTRMISSING, UNKNOWNATTR,
 
-struct range {
-  int first_byte;
-  int last_byte;
-  int bytes_covered;
-  int is_assigned;
-  bool *resources;
-  uerr_t status_least_severe;
-};
+  WARC_ERR, WARC_TMP_FOPENERR, WARC_TMP_FWRITEERR
+} uerr_t;
 
 /* 2005-02-19 SMS.
    Select an appropriate "orig" suffix and a separator character for
